package wmo

import (
	"context"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

func (c *Coll[TData]) FindOneAndUpdate(ctx context.Context, filterQuery bson.M, updateQuery bson.M) (TData, error) {
	mongoRes := c.coll.FindOneAndUpdate(ctx, filterQuery, updateQuery, options.FindOneAndUpdate().SetReturnDocument(options.After))

	return c.decodeSingle(ctx, mongoRes)
}

func (c *Coll[TData]) UpdateOne(ctx context.Context, filterQuery bson.M, updateQuery bson.M) error {
	_, err := c.coll.UpdateOne(ctx, filterQuery, updateQuery)
	if err != nil {
		return err
	}

	return nil
}

func (c *Coll[TData]) UpdateOneByID(ctx context.Context, id EntityID, updateQuery bson.M) error {
	_, err := c.coll.UpdateOne(ctx, bson.M{"_id": id}, updateQuery)
	if err != nil {
		return err
	}

	return nil
}

func (c *Coll[TData]) UpdateMany(ctx context.Context, filterQuery bson.M, updateQuery bson.M) (*mongo.UpdateResult, error) {
	res, err := c.coll.UpdateMany(ctx, filterQuery, updateQuery)
	if err != nil {
		return nil, err
	}

	return res, nil
}

func (c *Coll[TData]) ReplaceOne(ctx context.Context, filterQuery bson.M, value TData) error {
	_, err := c.coll.UpdateOne(ctx, filterQuery, bson.M{"$set": value})
	if err != nil {
		return err
	}

	return nil
}

func (c *Coll[TData]) FindOneAndReplace(ctx context.Context, filterQuery bson.M, value TData) (TData, error) {
	mongoRes := c.coll.FindOneAndUpdate(ctx, filterQuery, bson.M{"$set": value}, options.FindOneAndUpdate().SetReturnDocument(options.After))

	return c.decodeSingle(ctx, mongoRes)
}
